#include "mex.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void mexFunction(	int nlhs,	mxArray *plhs[],
						int nrhs,	const mxArray *prhs[])
{
	/******
	 * imwrite_raw12(fname, im)
	 * write an image specified in im 
	 * as a raw 12 bit/pixel monochrome image file
	 * to the filename given in fname
	 */
	char *pFileName;
	const unsigned short *pBuffer16;
	unsigned char *pBuffer8;
	FILE *fp;
	const mxArray *mxFileName;
	const mxArray *mxImageMat;
	int nFileName;
	unsigned nWords, nBytes, n;
	unsigned short w1, w2;

	/* check number of arguments */
	if(nrhs != 2)
	{
		mexErrMsgTxt("Wrong number of input arguments");
	}
	mxFileName	= prhs[0];
	mxImageMat	= prhs[1];

	/* parse filename argument */
	if(!mxIsClass(mxFileName, "char"))
	{
		mexErrMsgTxt("imwrite_raw12(fname, im): fname should be a filename");
		return;
	}
	nFileName	= mxGetNumberOfElements(mxFileName);
	pFileName	= (char*)malloc((nFileName+1) * sizeof(char));
	if(mxGetString(mxFileName, pFileName, nFileName+1))
	{
		mexErrMsgTxt("imwrite_raw12(fname, im): fname should be a filename");
		return;
	}

	/* parse image argument */
	if(!mxIsNumeric(mxImageMat) 
		|| !mxIsClass(mxImageMat, "uint16")
		|| mxGetNumberOfDimensions(mxImageMat) != 2)
	{
		mexErrMsgTxt("imwrite_raw12(fname, im): im should be a Nx x Ny matrix of uint16");
		return;
	}
	pBuffer16	= (const unsigned short*)mxGetData(mxImageMat);
	nWords		= mxGetM(mxImageMat) * mxGetN(mxImageMat);
	nBytes		= (nWords * 3) / 2;


	/* attempt to open file */
	fp	= fopen(pFileName, "wb");
	if(!fp)
	{
		mexErrMsgTxt("imwrite_raw12: unable to open file");
		return;
	}
	
	/* write to file */
	pBuffer8	= (unsigned char*) malloc(nBytes * sizeof(unsigned char));
	for(n = 0; n < nWords; n+=2)
	{
		w1		= pBuffer16[n+0];
		w2		= pBuffer16[n+1];

		pBuffer8[(n*3)/2 + 0] = (unsigned char)(w1 >> 4); // 8 MSB of word 1
		pBuffer8[(n*3)/2 + 1] = (unsigned char)(((w1 & 0x0F)<<4) | ((w2>>8)&0x0F)); // 4 LSB of word 1, 4 MSB of word 2
		pBuffer8[(n*3)/2 + 2] = (unsigned char)(w2 & 0xFF); // 8 LSB of word 2
	}
	fwrite(pBuffer8, sizeof(unsigned char), nBytes, fp);
	fclose(fp);

	/* free memory */
	free(pFileName);
	free(pBuffer8);

	return;
}
